//--------------------------------------------------------------
// Press F1 to get help about using script.
// To access an object that is not located in the current class, start the call with Globals.
// When using events and timers be cautious not to generate memoryleaks,
// please see the help for more information.
//---------------------------------------------------------------

namespace Neo.ApplicationFramework.Generated
{
    using System.Windows.Forms;
    using System;
    using System.Drawing;
    using Neo.ApplicationFramework.Tools;
    using Neo.ApplicationFramework.Common.Graphics.Logic;
    using Neo.ApplicationFramework.Controls;
    using Neo.ApplicationFramework.Interfaces;
    
	using Neo.ApplicationFramework.Tools.OpcClient;				// Used for tag access
	using System.Reflection;
	
	using System.IO.Ports;    									// Used for serial com

	public partial class scmSerMon
	{
		private SerialPort port;								// Communication port object
		
		private byte[] IndataBuffer = new byte[1000];			// Indatabuffer
		private int Inbuffptr = 0;								// Pointer used to fill Indatabuffer

		/// <summary>
		/// Method to fetch tag object by stringname
		/// </summary>
		/// <param name="propertyName">Tag Name</param>
		/// <returns>Tag object</returns>
		private GlobalDataItem GetGlobalDataItem(string propertyName)
		{
			PropertyInfo tagProperty = typeof(Neo.ApplicationFramework.Generated.Tags).GetProperty(propertyName);
			if(tagProperty == null)
				return null;
			else
				return tagProperty.GetValue(Globals.Tags, null) as GlobalDataItem;
		}		
		
		/// <summary>
		/// This method sets up all communication when the script module gets created
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// <returns></returns>
		void scmSerMon_Created(System.Object sender, System.EventArgs e)
		{
			string serialport = "COM1";								// Default serial port (on panel)
			if(Environment.OSVersion.Platform != PlatformID.WinCE)
				serialport = "COM9";								// Serial port (on PC)
			port = new SerialPort(serialport, 9600, Parity.None, 8, StopBits.One);			
			port.DataReceived += new SerialDataReceivedEventHandler(comPort_DataReceived);
			port.Open();
			port.Write(Environment.NewLine + "# Serial Tag Monitor" + Environment.NewLine +
				"# Use Tagname=Value to set tag or only Tagname to read value," + Environment.NewLine + 
				"# only numerical values are supported." + Environment.NewLine + 
				"# Remember that tag names are case sensitive" + Environment.NewLine);
		}		
		
		/// <summary>
		/// This metod is triggered when data is availible on the serial port, it handles
		/// recieving characters and adding them to the inbufffer. 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		/// <returns></returns>
		private	void comPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
			//retrieve number of bytes in the buffer
			int bytes = port.BytesToRead;			
			byte c = 0;
			
			//create a byte array to hold the awaiting data
			byte[] comBuffer = new byte[bytes+100];
			
			//read the data and store it
			port.Read(comBuffer, 0, bytes);
			
			//Copy the data to the indata buffer
			for(int i=0;i<bytes;i++)
			{
				c = comBuffer[i];					
				if (c!=13)									// If this is a termination character
				{		
					if(c>32)								// Strip command characters and spaces
					{
						if(Inbuffptr<IndataBuffer.Length)	// If Indatabuffer is not full
							IndataBuffer[Inbuffptr++] = c;	// Store recieved character in the indata buffer
						else             					// If Indata buffer is full
							Inbuffptr=0;					// Reset buffer pointer, throw all data
					}
				}
				else										// Termination character is found 
				{
					// Parse the incoming string
					try
					{
						ParseCmdLine(System.Text.Encoding.Default.GetString(IndataBuffer,0,Inbuffptr));
						Inbuffptr=0;						// Reset buffer pointer						
					}
					catch
					{
						port.Write("# Unable to parse command" + Environment.NewLine);
					}
				}
			}
		}
		
		/// <summary>
		/// It's here the inbuffer gets parsed and is acted on.
		/// Set a tag value in the application with Tagname=value[CR/LF]
		/// Get a tag value in the application with Tagname[CR/LF]
		/// If a line contains a #, it'll be ignored. All spaces are stripped.
		/// </summary>
		/// <param name="cmdstring"></param>
		/// <returns></returns>
		private void ParseCmdLine(string cmdstring)
		{
			string [] cmdsplit;
			double setvalue;
			string sendstring;
			
			// If the string contains anything and does not contain an ignore character
			if( (cmdstring.Length>0)  && (cmdstring.Contains("#") == false) )
			{
				cmdsplit = cmdstring.Split('=');		// Split string into tagname and value (if any)					
				// Check if the command was a tag value assigment or a value request
				if(cmdsplit.Length>1)					// If this is a tag value assignment
				{											
					try 
					{
						// Fetch the value to assign to the tag
						setvalue = Convert.ToDouble(cmdsplit[1]);
						// Set the value above to the correct tag
						GetGlobalDataItem(cmdsplit[0]).SetAnalog(setvalue);								
					}
					catch 
					{
						port.Write("# Invalid format/tag : " + cmdstring + Environment.NewLine);
					}
				}
				else								// If this is a value request
				{
					try 
					{
						// Tell te tag server to reftresh the value
						GetGlobalDataItem(cmdsplit[0]).Read();
						// Fetch the tag value and create a string
						sendstring = GetGlobalDataItem(cmdsplit[0]).Value.ToString();									
						port.Write(cmdsplit[0]+"=" + sendstring + Environment.NewLine);
					}
					catch
					{
						port.Write("# Invalid tag : " + cmdstring + Environment.NewLine);
					}
				}
			}					
		}	
    }
}
